import threading
import logging
import time
import numpy

class ResettableTimer(threading.Thread):
    """ Timer
    """
    def __init__(self,fPeriod, fctExpirationFunc, iRepeatMax = numpy.Inf):
        """ timer for periodic function calls
        use : 
        t = ResettableTimer(2.,myFunc)
        t.Pause()
        t.SetPeriod(0.5)
        t.Resume()
        t.Kill()
        
        Args:
            fPeriod(sec) : timer interval in seconds
            fctExpirationFunc(func) : function to be called on timer expiration
            iRepeatMax (int) : max number of expiration function call
        
        """
        threading.Thread.__init__ (self, target=self.run)
         
        self.fPeriod = fPeriod
        self.iRepeatMax = iRepeatMax
        self.fctExpirationFunc = fctExpirationFunc
        self._stopevent = threading.Event()
        self._pauseevent = threading.Event()

        self.pause_cond = threading.Condition(threading.Lock())

        self.bPaused = False
        self.start()
            
    def run (self):
        """ Main loop
        """
        logging.debug("utildate::ResettableTimer Run fPeriod : %f"%self.fPeriod)
        iRepeat = 0
        
        while not self._stopevent.isSet() :
            with self.pause_cond:
                if self._pauseevent.isSet(): 
                    self.bPaused = True
                    self.pause_cond.wait()
            
            self.bPaused = False
            time.sleep(self.fPeriod)

            if not self._stopevent.isSet() and not self._pauseevent.isSet():
                try:
                    self.fctExpirationFunc()
                except:
                    logging.exception("ResettableTimer::run unexpected error in timed function")
                
                iRepeat += 1
                if iRepeat >= self.iRepeatMax : self._stopevent.set()
                
            if   self._pauseevent.isSet()and not self._stopevent.isSet():
                #delayed time
                time.sleep(0.1)
            
                
    def Pause(self):
        logging.debug("utildate::ResettableTimer Pause")
        if self._pauseevent.isSet() : return 
        else:       
            self._pauseevent.set()
            self.pause_cond.acquire()

    def IsPaused(self):
        return self.bPaused
     
    def Resume(self):
        logging.debug("utildate::ResettableTimer Resume")
        if not self._pauseevent.isSet() : return
                
        self._pauseevent.clear()
        self.pause_cond.notify()
        self.pause_cond.release()

    def Kill(self):
        logging.debug("utildate::ResettableTimer Kill")
        self._stopevent.set()
        if self._pauseevent.isSet() : self.Resume() #release the lock
        

    def GetState(self):
        return not self._pauseevent.isSet()
    
    def SetPeriod(self,fPeriod):
        self.fPeriod = fPeriod